﻿//**************************************************************
// C# .NET PDF417 Class Font Encoder for IDAutomation Barcode 
// Fonts V.2020
// © Copyright, 2020 IDAutomation.com, Inc. All rights reserved.
// This function returns a text string as an Expression. This
// string will generate the barcode when combined with the
// correct barcode font. 
// Font to use: IDAutomation2D 
//
//*****************************************************************
// Limitations of the Demo Version:
// The demo version of this product includes a fully
// functional 2D font and is designed only for the
// purpose of testing the font in various applications
// and point sizes. To generate dynamic symbols,
// a font encoder is required to format the data being
// encoded into the 2D symbol.
// Only static data can be provided for the encoders
// in the demo version, because providing a working
// encoder would reveal the complete source code,
// which is only available in the licensed version.
// To order, please visit:
// https://www.idautomation.com/
//*****************************************************************
//
// Please refer to the User Manual and Included Examples
// https://www.idautomation.com/barcode-components/dotnet-standard/
//
//*****************************************************************


using System;
using System.Globalization;
using System.Text;

namespace IDAutomation.NetStandard.PDF417.FontEncoder
{
	/// <summary>
	/// Defines all encoding algorithms and creates a binary bitmap representation of the PDF417 symbol.
	/// </summary>
	public class PDF417
	{
		private int rows = 0;
		private int cols = 0;
		private string[] ResultRows;
		private String _newline = "";
		private int[][] Matrix;


		// Macro PDF variables 
		/// <summary>
		/// Internal variable set by the bMacroPDFEnable property
		/// </summary>
		private bool bMacroPDFEnable = false;
		/// <summary>
		/// Internal variable set by the iMacroPDFSegmentIndex property
		/// </summary>
		private int iMacroPDFSegmentIndex = 0;
		/// <summary>
		/// Internal variable set by the iMacroPDFFileID property
		/// </summary>
		private int iMacroPDFFileID = 0;
		/// <summary>
		/// Internal variable set by the bMacroPDFLastSegment property
		/// </summary>
		private bool bMacroPDFLastSegment = false;

		/// <summary>
		/// Number of rows in the PDF417 barcode
		/// </summary>
		private int numberPDF417Rows = 0;

		/// <summary>
		/// Number of columns in the PDF417 barcode
		/// </summary>
		private int numberPDF417Columns = 0;


		/// <summary>
		/// number of rows of the PDF417 code. If 0 the number will be calculated automatically.
		/// </summary>
		private int runtimeRows = 0;

		/// <summary>
		/// Caclulated Code Words
		/// </summary>
		private int[] iarrCodeWords = null;

		/// <summary>
		/// An error message that can be written to the output graphic if there is a problem with the input data or settings
		/// </summary>
		private string errorDescription = "";


		/// <summary>
		/// This function encodes the data passed in into a suitable format so that the barcode font can be applied to the data.
		/// </summary>
		/// <param name="DataToEncode">The data to be encoded in the barcode.</param>
		/// <param name="ApplyTilde">Determines if data after the tilde character should be interpreted as an ASCII code.</param>
		/// <param name="EccLevel">The level of error correction used to create the barcode.</param>
		/// <param name="PDFMode">The mode of compaction used to encode data in the symbol.</param>
		/// <param name="NumColumns">The number of columns in the PDF417 barcode.</param>
		/// <param name="NumRows">The number of rows in the PDF417 barcode.</param>
		/// <param name="Truncate">The right hand side of the PDF417 is removed or truncated.</param>
		/// <param name="QuietZone">Define the size of a blank zone around the code.</param>
		/// <param name="ModuleSize">Define the size of each module.</param>
		/// <returns>A array representation of raster PDF417's image.</returns>
		public byte[] EncodePDF417(string DataToEncode, bool ApplyTilde, int EccLevel, EncodingModes PDFMode,int NumColumns, int NumRows, bool Truncate,  int QuietZone = 2, int ModuleSize = 2)
		{



			rows = 24;
			cols = 121;

			Matrix = new int[24][];
			Matrix[0] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[1] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[2] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[3] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[4] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[5] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[6] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[7] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[8] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[9] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[10] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[11] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[12] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[13] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[14] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[15] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[16] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[17] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[18] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[19] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[20] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[21] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[22] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[23] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };


			byte[] Result = null;

			int k;
			int intPad = 0;
			int pixels = 0;
			int bit = 0;
			int bWidth = 0;

			bWidth = ((int)((cols * ModuleSize) / 8) + ((cols * ModuleSize) % 8 > 0 ? 1 : 0));
			int intTemp = bWidth % 4;
			if (intTemp != 0)
			{
				intPad = 4 - intTemp;
			}

			Result = new byte[64 + rows * ModuleSize * (bWidth + intPad)];

			Result[0] = 0x42;
			Result[1] = 0x4d;
			for (k = 0; k < 8; k++)
			{
				Result[2 + k] = 0;
			}
			Result[10] = 0x3E;
			for (k = 0; k < 3; k++)
			{
				Result[11 + k] = 0;
			}
			Result[14] = 0x28;
			for (k = 0; k < 3; k++)
			{
				Result[15 + k] = 0;
			}

			// width
			Result[18] = (byte)((cols * ModuleSize) % 256);
			Result[19] = (byte)(Math.Floor((double)(cols * ModuleSize) / 256));
			Result[20] = 0;
			Result[21] = 0;

			// height
			Result[22] = (byte)((rows * ModuleSize) % 256);
			Result[23] = (byte)(Math.Floor((double)(rows * ModuleSize) / 256));
			Result[24] = 0; ;
			Result[25] = 0;

			Result[26] = 1;
			Result[27] = 0;
			Result[28] = 1;
			Result[29] = 0;
			for (k = 0; k < 8; k++)
			{
				Result[30 + k] = 0;
			}
			Result[38] = 0x25;
			Result[39] = 0x0e;
			Result[40] = 0;
			Result[41] = 0;

			Result[42] = 0x25;
			Result[43] = 0x0e;
			Result[44] = 0;
			Result[45] = 0;

			for (k = 0; k < 8; k++)
			{
				Result[46 + k] = 0;
			}

			for (k = 0; k < 3; k++)
			{
				Result[54 + k] = 0xff;
			}
			for (k = 0; k < 5; k++)
			{
				Result[57 + k] = 0;
			}

			int pos = 62;
			for (int y = (rows) - 1; y >= 0; y--)
			{



				for (int m = 0; m < ModuleSize; m++)
				{
					pixels = 0;
					bit = 8;

					for (int x = 0; x < (cols); x++)
					{
						if ((Matrix[y][x] != 0))
						{

							for (int l = 0; l < ModuleSize; l++)
							{
								bit = bit - 1;
								pixels = pixels + (int)Math.Pow(2, bit);
								if (bit == 0)
								{
									Result[pos] = (byte)pixels;
									// row +1 and row+2 are ugual
									//										Result[pos + (bWidth + intPad)] = (byte)pixels;
									//										Result[pos + 2*(bWidth + intPad)] = (byte)pixels;

									pos++;
									bit = 8;
									pixels = 0;
								}
							}

						}
						else
						{

							for (int l = 0; l < ModuleSize; l++)
							{
								bit = bit - 1;
								if (bit == 0)
								{
									Result[pos] = (byte)pixels;
									// row +1 and row+2 are ugual
									//										Result[pos + (bWidth + intPad)] = (byte)pixels;
									//										Result[pos + 2 * (bWidth + intPad)] = (byte)pixels;
									pos++;
									bit = 8;
									pixels = 0;
								}
							}



						}
					}
					if (bit != 8)
					{
						Result[pos] = (byte)pixels;
						// row +1 and row+2 are ugual
						//							Result[pos + (bWidth + intPad)] = (byte)pixels;
						//							Result[pos + 2 * (bWidth + intPad)] = (byte)pixels;
						pos++;
					}
					pos = pos + intPad; //+ 2 * (bWidth + intPad);
				}



			}

			return Result;


		}


		/// <summary>
		/// This function encodes the data passed in into a suitable format so that the barcode font can be applied to the data.
		/// </summary>
		/// <param name="DataToEncode">The data to be encoded in the barcode.</param>
		/// <param name="ApplyTilde">Determines if data after the tilde character should be interpreted as an ASCII code.</param>
		/// <param name="EccLevel">The level of error correction used to create the barcode.</param>
		/// <param name="PDFMode">The mode of compaction used to encode data in the symbol.</param>
		/// <param name="NumColumns">The number of columns in the PDF417 barcode.</param>
		/// <param name="NumRows">The number of rows in the PDF417 barcode.</param>
		/// <param name="Truncate">The right hand side of the PDF417 is removed or truncated.</param>
		/// <param name="VectorType"> Define the type of vector image for output.</param>
		/// <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
		/// <param name="QuietZone">Define the size of a blank zone around the code.</param>
		/// <param name="ModuleSize">Define the size of each module.</param>
		/// <returns>A string representation of vector PDF417's image.</returns>
		public string EncodePDF417(string DataToEncode, bool ApplyTilde, int EccLevel, EncodingModes PDFMode, int NumColumns, int NumRows, bool Truncate, VectorTypes VectorType = VectorTypes.VecEPS, string NewLine = null, int QuietZone = 2, int ModuleSize = 2)
		{

			//allows user to set an end-of-line (EOL) string
			if (NewLine == null)
			{
				NewLine = Environment.NewLine; //default
			}

			_newline = NewLine;


			rows = 24;
			cols = 121;

			Matrix = new int[24][];
			Matrix[0] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[1] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[2] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[3] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[4] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[5] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[6] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[7] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[8] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[9] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[10] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[11] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[12] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[13] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[14] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[15] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[16] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[17] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[18] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[19] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[20] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[21] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[22] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };
			Matrix[23] = new int[] { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0 };

			StringBuilder strOut = new StringBuilder("");
			string tempW = "";
			string tempH = "";
			ModuleSize = ModuleSize * 4;
			tempW = (cols * ModuleSize).ToString();
			tempH = (rows * ModuleSize).ToString();

			if (VectorType == VectorTypes.VecSVG)
			{


				strOut.Append("<?xml version='1.0' standalone='no'?>" + _newline);
				strOut.Append("<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>" + _newline);
				strOut.Append("<svg width='" + tempW + "' height='" + tempH + "' version='1.1' xmlns='http://www.w3.org/2000/svg'>" + _newline);
				strOut.Append("  <g id='pdf417'>" + _newline);
				strOut.Append("    <g id='background' fill='#ffffff' stroke='#ffffff'>" + _newline);
				strOut.Append("      <rect x='0' y='0' width='" + tempW + "' height='" + tempH + "' />" + _newline);
				strOut.Append("    </g>" + _newline);
				strOut.Append("    <g id='pattern' fill='#000000' stroke='#000000'>" + _newline);

				for (int y = 0; y < rows; y++)
				{
					for (int x = 0; x < cols; x++)
					{
						if (Matrix[y][x] != 0)
						{
							strOut.Append("    <g id='R" + y + "C" + x + "' >" + _newline);
							strOut.Append("      <rect x='" + (x * ModuleSize).ToString() + "' y='" + (y * ModuleSize).ToString() + "' width='" + (ModuleSize).ToString() + "' height='" + (ModuleSize ).ToString() + "' />" + _newline);
							strOut.Append("    </g>" + _newline);
						}
					}
				}
				strOut.Append("    </g>" + _newline);
				strOut.Append("  </g>" + _newline);
				strOut.Append("</svg>" + _newline);
			}
			else if (VectorType == VectorTypes.VecEPS)
			{

				strOut.Append("%!PS-Adobe-3.0 EPSF-3.0" + _newline);
				strOut.Append("%%Creator: IDAutomation.com" + _newline);
				strOut.Append("%%Title: pdf417" + _newline);
				strOut.Append("%%CreationDate: " + DateTime.Now.ToString("ddd mmm d hh:mm:ss yyyy") + _newline);
				strOut.Append("%%BoundingBox: 0 0 " + tempW + " " + tempH + _newline);
				strOut.Append("%%DocumentData: Clean7Bit" + _newline);
				strOut.Append("%%EndComments" + _newline);
				strOut.Append("%%BeginProlog" + _newline);
				strOut.Append("/bd { bind def } bind def" + _newline);
				strOut.Append("/incompound false def" + _newline);
				strOut.Append("/m { moveto } bd" + _newline);
				strOut.Append("/l { lineto } bd" + _newline);
				strOut.Append("/*U { /incompound false def f} bd" + _newline);
				strOut.Append("/k { setcmykcolor } bd" + _newline);

				strOut.Append("/c { curveto } bd" + _newline);
				strOut.Append("/F { incompound not {fill} if } bd" + _newline);
				strOut.Append("/f { closepath F } bd" + _newline);
				strOut.Append("/S { stroke } bd" + _newline);
				strOut.Append("/*u { /incompound true def } bd" + _newline);
				strOut.Append("/K { k } bd" + _newline);

				strOut.Append("%%EndProlog" + _newline);
				strOut.Append("%%BeginSetup" + _newline);
				strOut.Append("%%EndSetup" + _newline);
				strOut.Append("0.000 0.000 0.000 0.000 k" + _newline);
				strOut.Append("0 0 m" + _newline);
				strOut.Append("0 0 l" + _newline);
				strOut.Append(tempW + " 0 l" + _newline);
				strOut.Append(tempW + " " + tempH + " l" + _newline);
				strOut.Append("0 " + tempH + " l" + _newline);
				strOut.Append("f" + _newline);
				strOut.Append("*U" + _newline);

				int n = rows * ModuleSize;
				strOut.Append("0.000 0.000 0.000 1.000 k" + _newline);
				for (int y = 0; y < rows; y++)
				{
					for (int x = 0; x < cols; x++)
					{
						if (Matrix[y][x] != 0)
						{
							strOut.Append((x * ModuleSize).ToString() + " " + (n - y * ModuleSize).ToString() + " m" + _newline);
							strOut.Append((x * ModuleSize).ToString() + " " + (n - y * ModuleSize).ToString() + " l" + _newline);
							strOut.Append(((x + 1) * ModuleSize).ToString() + " " + (n - y * ModuleSize).ToString() + " l" + _newline);
							strOut.Append(((x + 1) * ModuleSize).ToString() + " " + (n - (y + 1) * ModuleSize).ToString() + " l" + _newline);
							strOut.Append(((x) * ModuleSize).ToString() + " " + (n - (y + 1) * ModuleSize).ToString() + " l" + _newline);
							strOut.Append("f" + _newline);
						}
					}
				}

				strOut.Append("*U" + _newline);
				strOut.Append("%%Trailer" + _newline);
				strOut.Append("%%EOF" + _newline);

			}
			else if (VectorType == VectorTypes.VecHTML)
			{
				strOut.Append("<div style='font-size:0;background-color:#ffffff;position:relative;width:" + tempW + "px;height:" + tempH + "px;'>" + _newline);
				for (int y = 0; y < rows; y++)
				{
					for (int x = 0; x < cols; x++)
					{
						if (Matrix[y][x] != 0)
						{
							strOut.Append("<div style='background-color:#000000;width:" + ModuleSize.ToString() + "px;height:" + (ModuleSize).ToString() + "px;position:absolute;left:" + (x * ModuleSize).ToString() + "px;top:" + (y * ModuleSize).ToString() + "px;'>&nbsp;</div>" + _newline);
						}
					}
				}

				strOut.Append("</div>" + _newline);
			}

			return strOut.ToString();

		}

		/// <summary>
		/// This function encodes the data passed in into a suitable format so that the barcode font can be applied to the data.
		/// </summary>
		/// <param name="DataToEncode">The data to be encoded in the barcode.</param>
		/// <param name="ApplyTilde">Determines if data after the tilde character should be interpreted as an ASCII code.</param>
		/// <param name="EccLevel">The level of error correction used to create the barcode.</param>
		/// <param name="PDFMode">The mode of compaction used to encode data in the symbol.</param>
		/// <param name="NumColumns">The number of columns in the PDF417 barcode.</param>
		/// <param name="NumRows">The number of rows in the PDF417 barcode.</param>
		/// <param name="Truncate">The right hand side of the PDF417 is removed or truncated.</param>
		/// <param name="OutputType"> Define the type of font for output.</param>
		/// <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
		/// <returns>Encoded text for use with the 2D barcode font.</returns>
		public string EncodePDF417(string DataToEncode, bool ApplyTilde, int EccLevel, EncodingModes PDFMode, int NumColumns, int NumRows, bool Truncate, OutputTypes OutputType = OutputTypes.IDA2DFont, string NewLine = null)
		{

			//allows user to set an end-of-line (EOL) string
			if (NewLine == null)
			{
				NewLine = Environment.NewLine; //default
			}

			switch (OutputType)
			{

				case OutputTypes.IDAMonospaceFont:
					return "████████ █ █ █   █████ █ █ █████  ███ █ ██  ████   ██    ██ ██   ██ █  ███ ███ █     █████ █ █ █████  ███████ █   █ █  █" + NewLine + "████████ █ █ █   █████▄▀▄▀▄▀▀▀██▄ █▀▀ █▄█▀ ▄▀███▄▄ ██    ██ ▀█▄▄▄▀█ █▄ █▀▀▄██▀▄▀     ████▀▄▀▄▀ █▀▀▀▀  ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ██████ █ █   ███ █   ███  █ █████ ██    ██  ████ █ ██ █  ███ █      ████ █ █  █      ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █ █ █  ████      ██  █████  █   █ █  ███   ██  ███ ███ ███ █   ████ ███ █ █   ██████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █▄▀▄▀▄▄██▀▀▄▄▄▄▄ █▀▄▄█▀▀▀█▄▄▀   █ █  ██▀▄▄▄██ ▄███ ██▀ ▀██▄▀▄▄ ▀▀▀█ ██▀▄▀▄█▄▄ ▀█████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ██ █ ████  █████ █ ███   ███    █ █  ██ █████ ████ ██   ███ ██    █ ██ █ ████  █████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███ █ ███    ██  █     █ ████ █   ████ ██    ██ █  █ ██████ █ ██    ████ █ ███  ███  ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███▄█ █▀█▄▄▄ ██  █▄   ▄█▄██▀▀▄▀ ▄ ██▀█ ██▄▄▄▄▀▀ █  █ ▀▀███▀ █▄▀█▄▄  ███▀▄▀▄███▄ █▀▀  ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █████ █ ████ ██  ██   █████  █  █ ██ █ ██████   █  █   ███  ██ ███  ███ █ █████ █    ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███ █  ███ █████ ███ ████ ██   █  ██ ██     █    █ ████ ██ █    ███ ███ █  ███ █████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███▄█ ▄▀▀█ ██▀▀▀ █▀▀▄██▀▀▄▀█▄▄▄█  ██ ▀▀ ▄ ▄▄█   ▄▀ ██▀▀ ▀█ █▄▄  ▀█▀ █▀█ █▄▄███ ███▀▀ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █████ █  █ ██    █  ███  █ █████  ██    █ ███   █  ██    █ ███   █  █ █ ██████ ███   ███████ █   █ █  █ " + NewLine + "";
				default:
					return "777777770707070007676724353544662075407375124766230760111660572235707316553674241111077765252407555510777777707000707007" + NewLine + "777777770707070007735347671154774071444131776212050732743755552647407422377607523114077361657731476640777777707000707007" + NewLine;
			}
		}

	}


	/// <summary>
	/// Various types of output.
	/// </summary>
	public enum OutputTypes
	{

		/// <summary>
		/// For use with IDAutomation PDF417 or 2D Barcode Fonts. Returns PDF417 encoded text. 
		/// </summary>
		IDA2DFont,


#if !ENCODER
		/// <summary>
		/// Returns PDF417 native object. For use with monospaced fonts.
		/// </summary>
		IDAMonospaceFont

#endif


	}


	/// <summary>
	/// Various types of vector image output.
	/// </summary>
	public enum VectorTypes
	{

		/// <summary>
		/// Returns a string with the SVG representation. 
		/// </summary>
		VecSVG,

		/// <summary>
		/// Return a string with the EPS representation.
		/// </summary>
		VecEPS,

		/// <summary>
		/// Return a string with the HTML representation.
		/// </summary>
		VecHTML,

	}

	/// <summary>
	/// Various modes used to Data represented in the symbol may be encoded by one of these types.
	/// </summary>
	public enum EncodingModes
	{
		/// <summary>
		/// In this mode data is encoded at 8 bits per character.
		/// </summary>
		Binary,

		/// <summary>
		/// Use a TEXT encoding chartset optimized for data that only contains alphanumeric characters. 
		/// </summary>
		Text,
	}
}


